unit MainFormU;

interface

uses
  Androidapi.JNI.GraphicsContentViewText,
  Androidapi.JNI.Nfc,
  Androidapi.JNI.Blong,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.StdCtrls, FMX.Layouts, FMX.ListBox, NFCHelper,
  FMX.Edit, FMX.Controls.Presentation;

type
  TMainForm = class(TForm)
    NFCTagIdLabel: TLabel;
    InfoLabel: TLabel;
    PromptLabel: TLabel;
    TagWriteEdit: TEdit;
    TagWriteButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TagWriteButtonClick(Sender: TObject);
  private
    { Private declarations }
    NfcAdapter: JNfcAdapter;
    Listener: JBlongListener;
  public
    { Public declarations }
    procedure OnNewIntent(Intent: JIntent);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.fmx}

uses
{$IF CompilerVersion >= 28}
  Androidapi.NativeActivity,
{$ENDIF}
  Androidapi.Helpers,
  Androidapi.JNIBridge,
  Androidapi.JNI.Os,
  Androidapi.JNI.Toast;

type
  TNativeActivitySubclassListener = class (TJavaLocal, JBlongListener)
  public
    procedure onNewIntent(intent: JIntent); cdecl;
  end;

{ TNativeActivitySubclassListener }

procedure TNativeActivitySubclassListener.onNewIntent(intent: JIntent);
begin
  TThread.Queue(nil,
    procedure
    begin
      MainForm.OnNewIntent(intent);
    end);
end;

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Log.d('OnCreate');
  Listener := TNativeActivitySubclassListener.Create;
{$IF CompilerVersion >= 28}
  TJNativeActivitySubclass.Wrap(SharedActivity).setListener(Listener);
{$ELSE}
  TJNativeActivitySubclass.Wrap(PANativeActivity(System.DelphiActivity)^.clazz).setListener(Listener);
{$ENDIF}
  NfcAdapter := TJNfcAdapter.JavaClass.getDefaultAdapter(SharedActivityContext);
  if NfcAdapter = nil then
  begin
    // Could do with exiting here maybe...
    raise Exception.Create('No NFC adapter present');
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  TNativeActivitySubclassListener(Listener).DisposeOf;
end;

procedure TMainForm.FormActivate(Sender: TObject);
var
  Intent: JIntent;
begin
  Log.d('OnActivate');
  if NfcAdapter <> nil then
  begin
    if not NfcAdapter.isEnabled then
    begin
      Toast('NFC is not enabled.' + LineFeed + 'Launching NFC settings.');
      SharedActivity.startActivity(TJIntent.JavaClass.init(StringToJString('android.settings.NFC_SETTINGS')));
    end;
  end;
  Intent := SharedActivity.getIntent;
  if not TJIntent.JavaClass.ACTION_MAIN.equals(Intent.getAction) then
  begin
    Log.d('Passing along received intent');
    OnNewIntent(Intent);
  end;
end;

procedure TMainForm.OnNewIntent(Intent: JIntent);
var
  TagParcel: JParcelable;
  Tag: JTag;
begin
  Log.d('TMainForm.OnNewIntent');
  SharedActivity.setIntent(Intent);
  if Intent <> nil then
  begin
    Log.d('Intent action = %s', [JStringToString(Intent.getAction)]);
    if TJNfcAdapter.JavaClass.ACTION_NDEF_DISCOVERED.equals(Intent.getAction) or
       TJNfcAdapter.JavaClass.ACTION_TECH_DISCOVERED.equals(Intent.getAction) or
       TJNfcAdapter.JavaClass.ACTION_TAG_DISCOVERED.equals(Intent.getAction) then
    begin
      PromptLabel.Visible := False;
      Log.d('Getting Tag parcel from the received Intent');
      TagParcel := Intent.getParcelableExtra(TJNfcAdapter.JavaClass.EXTRA_TAG);
      if TagParcel <> nil then
      begin
        Log.d('Wrapping tag from the parcel');
        Tag := TJTag.Wrap(TagParcel);
      end;
      InfoLabel.Text := '';
      NFCTagIdLabel.Text := HandleNfcTag(Tag,
        procedure (const Msg: string)
        begin
          Log.d('Adding to UI: ' + Msg);
          InfoLabel.Text := InfoLabel.Text + Msg + LineFeed;
        end);
    end;
  end;
end;

procedure TMainForm.TagWriteButtonClick(Sender: TObject);
var
  TagParcel: JParcelable;
  Tag: JTag;
  Intent: JIntent;
begin
  if (NfcAdapter <> nil) and NfcAdapter.isEnabled then
  begin
    Intent := SharedActivity.getIntent;
    TagParcel := Intent.getParcelableExtra(TJNfcAdapter.JavaClass.EXTRA_TAG);
    if TagParcel <> nil then
    begin
      Log.d('Wrapping tag from the parcel');
      Tag := TJTag.Wrap(TagParcel);
      if not WriteTagText(TagWriteEdit.Text, Tag) then
        raise Exception.Create('Error connecting to tag');
    end;
  end
  else
    raise Exception.Create('NFC is not available');
end;

end.
